#include "command.h"
#include "armscope_cmd.h"
#include "armscope_settings.h"
#include "buzzer.h"
#include "board.h"
#include "board_armscope.h"
#include "cdc_enumerate.h"
#include "usb.h"
#include "timer.h"
#include "delay.h"


enum E_ACTION_REQUEST ExecuteCmd(char *dataIn, int length,
	struct S_ARMSCOPE_SETTINGS * const settings)
{
	//sprintf (string, "kod: %d ", (int)dataIn[2]);
	//siprintf(message,"-I- Len %d:\n\r",length);
	//pCDC.Write(&pCDC, string, 10);
	if (dataIn[0] == CMD_FUNCTION)	//special functions
	{
		switch (dataIn[1])
		{
		case CMD_FUNCTION_IDENTIFY:		//identify request
			beep();
			pCDC.Write(&pCDC, "VDSO2 20090921", 15);
			return REQUEST_RESTART;
			break;
		case CMD_FUNCTION_COUPLING_DC:		//DC
			beep();
			settings->coupling_type = ARMSCOPE_COUPLING_DC;
			setACDC();
			break;
		case CMD_FUNCTION_COUPLING_AC:		//AC
			beep();
			settings->coupling_type = ARMSCOPE_COUPLING_AC;
			clearACDC();
			break;
		case CMD_FUNCTION_TRIGGER_SLOPE_FALLING:		//internal trigger, falling slope
			beep();
			settings->trigger_type = ARMSCOPE_TRIGGER_SLOPE_FALLING;
			break;
		case CMD_FUNCTION_TRIGGER_SLOPE_RAISING:		//internal trigger, raising slope
			beep();
			settings->trigger_type = ARMSCOPE_TRIGGER_SLOPE_RAISING;
			break;
		case CMD_FUNCTION_TRIGGER_MANUAL:		//manual trigger
			beep();
			settings->trigger_type = ARMSCOPE_TRIGGER_MANUAL;
			break;
		case CMD_FUNCTION_TRIGGER_EXECUTE:		//trig now
			return REQUEST_TRIGGER;
			break;
		case CMD_FUNCTION_FETCH_DATA:		//transfer request
			return REQUEST_DATA_FETCH;
			break;
		}
	}
	
	if (dataIn[0] == CMD_SET_SAMPLING_FREQ)	//sampling frequency
	{
		beep();
		SAMPLING_FREQUENCY = E_ARMSCOPE_SAMPLING_MAX;	
		while (dataIn[1]>0)
		{
			dataIn[1]--;
			SAMPLING_FREQUENCY /= 2;
		}
		settings->sampling_freq = SAMPLING_FREQUENCY;
		/** \note timer.h: #define   TIMER_VALUE            (MCK/(2*SAMPLING_FREQUENCY))
		*/
		//if(SAMPLING_FREQUENCY == 600000) lbeep();
		AT91F_TC_Open ( AT91C_BASE_TC0,
						(AT91C_TC_ASWTRG_CLEAR | AT91C_TC_ACPC_CLEAR | AT91C_TC_ACPA_SET | AT91C_TC_LDRA_RISING |
						 AT91C_TC_WAVE  | AT91C_TC_WAVESEL_UP_AUTO | TC_MCK_DIV ),
						TIMER_ID);
	}
	if (dataIn[0] == CMD_SET_CAPTURE_BUFFER_SIZE)	//sterowanie wielkoscia bufora akwizycji
	{
		beep();
		switch (dataIn[1])
		{
		case 0x00:
			settings->capture_buf_size = 512;
			break;
		case 0x01:
			settings->capture_buf_size = 1024;
			break;
		case 0x02:
			settings->capture_buf_size = 2048;
			break;
		case 0x03:
			settings->capture_buf_size = 4096;
			break;
		case 0x04:
			settings->capture_buf_size = 8192;
			break;
		case 0x05:
			settings->capture_buf_size = 14336;
			break;
		}
	}
	
	if (dataIn[0] == CMD_SET_TRIGGER_LEVEL)	//sterowanie poziomem wyzwalania
	{
		beep();
		settings->trigger_level = dataIn[1];
	}

	if (dataIn[0] == CMD_SET_SENSITIVITY)	//zmiana czulosci
	{
		beep();
		switch (dataIn[1])
		{
		case 0x01:		// highest sensitivity
			clearX10();	// no 1:10 relay
			clearG1();
			clearG2();
			setG3();
			break;
		case 0x02:
			clearX10();
			clearG1();
			setG2();
			clearG3();
			break;
		case 0x03:
			clearX10();
			setG1();
			clearG2();
			clearG3();
			break;
		case 0x04:
			clearX10();
			clearG1();
			clearG2();
			clearG3();
			break;
		case 0x05:
			setX10();
			setG1();
			clearG2();
			clearG3();
			break;
		case 0x06:
			setX10();
			clearG1();
			clearG2();
			clearG3();
			break;
		}
		if (dataIn[1] >= 0x01 && dataIn[1] <= 0x06)
			settings->sensitivity = dataIn[1];
		// give some time for ADC front-end
		Delay1k(500);
	}
	
	return REQUEST_NONE;
}
